
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

# == Schema Information
#
# Table name: submissions

# t.string   "justification"
# t.integer  "status",             default: 0
# t.datetime "answered_at"
# t.integer  "curator_id"
# t.integer  "submitter_id",                   null: false
# t.integer  "learning_object_id"
# t.datetime "created_at",                     null: false
# t.datetime "updated_at",                     null: false

class Submission < ApplicationRecord
  include Trackable

  enum status: [:submitted, :rejected, :accepted]

  belongs_to :submitter, class_name: "User"
  belongs_to :curator, class_name: "User"
  belongs_to :learning_object
  has_many :curator_assignments, dependent: :destroy
  has_many :assignees, through: :curator_assignments, source: :user
  has_many :answers, dependent: :destroy
  has_many :questions, through: :answers

  validates :submitter, :learning_object, presence: true
  validates :learning_object_id, uniqueness: true

  after_update :update_status

  def update_status
    if curator_id_changed?
      curator_assignments.each do |ca|
        if ca.user == curator
          ca.answered!
        else
          ca.ignored!
        end
      end

      status = answers.all? { |a| a.accepted? } ? :accepted : :rejected
      self.update_columns(status: status, answered_at: Time.current)
      new_activity(self.status)
    end
  end

  def new_update_activity
    nil
  end

  def activity_privacy
    'private'
  end

  def activity_recipient
    learning_object
  end

  def activity_owner
    submitter
  end
end
