
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class LearningObjectPolicy < ApplicationPolicy
  include SociablePolicy
  include ReportablePolicy
  include TaggablePolicy
  include SubjectablePolicy
  include StageablePolicy

  class Scope < Scope
    def resolve
      if user.nil?
        scope.where(state: 'published')
      elsif user_can_edit?
        scope.all
      else
        scope.where(state: 'published')
      end
    end
  end

  def create?
    record if user_can_create?
  end

  def update?
    record if ( user_can_update? && record.state == 'draft' ) || ( user.is_publisher? && user_can_update? )
  end

  def publish?
    record if user_can_publish?
  end

  def destroy?
    record if owns?
  end

  def destroy_attachment?
    record if owns?
  end

  def remove_thumbnail?
    record if owns?
  end

  def add_infohash?
    record if user.is_admin?
  end

  def add_attachment_id?
    record if user.is_admin?
  end

  def show?
    return record if record.published? || ( !user.nil? && (user_can_edit? || user.is_curator?))
    return record if user == record.publisher
  end

  def index?
    record
  end

  def add_object?
    record if user_exists?
  end

  def remove_object?
    record if user_exists?
  end

  def owner
    record.publisher
  end

end
