
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require_dependency 'dspace'

class DeleteBitstreamWorker
  include Sidekiq::Worker
  sidekiq_options queue: :dspace

  def perform(attachment_id = nil)
    attachment = LearningObject::Attachment.where(id: attachment_id).first
    return false if attachment.blank? || !wait_bitstream(attachment)

    # Require dspace gem
    Bundler.require(*Rails.groups)

    DspaceService.create_client.bitstreams.delete(id: attachment.id_dspace)
  ensure
    ActiveRecord::Base.clear_active_connections!
  end

  private

  def wait_bitstream(attachment)
    if attachment.id_dspace.blank?
      Timeout.timeout(60) do
        sleep(1.0) while attachment.id_dspace.blank?
      end
    end
    true
  rescue Timeout::Error
    false
  end
end
